@@ -28,6 +28,7 @@ package beast.app;
 
 import beast.core.*;
 import beast.core.util.Log;
+import beast.util.BEASTClassLoader;
 import beast.util.PackageManager;
 
 import java.io.*;
@@ -102,7 +103,7 @@ public class DocMaker {
         while (iter.hasNext()) {
             String beastObjectName = iter.next();
             try {
-                Class<?> _class = Class.forName(beastObjectName);
+                Class<?> _class = BEASTClassLoader.forName(beastObjectName);
                 BEASTObject beastObject = (BEASTObject) _class.newInstance();
                 String description = getInheritableDescription(beastObject.getClass());
                 Log.warning.println(beastObjectName + " => " + description);
@@ -137,7 +138,7 @@ public class DocMaker {
     void makeJavaDoc() {
         for (String beastObjectName : m_beastObjectNames) {
             try {
-                BEASTObject beastObject = (BEASTObject) Class.forName(beastObjectName).newInstance();
+                BEASTObject beastObject = (BEASTObject) BEASTClassLoader.forName(beastObjectName).newInstance();
                 Log.info.println(beastObjectName + ":@description:" + beastObject.getDescription());
                 for (Input<?> input : beastObject.listInputs()) {
                     Log.info.println(beastObjectName + ":" + input.getName() + ":" + input.getTipText());
@@ -330,7 +331,7 @@ public class DocMaker {
         List<String> implementations = new ArrayList<>();
         for (String beastObjectName : m_beastObjectNames) {
             try {
-                if (!beastObjectName.equals(name) && beastObject.getClass().isAssignableFrom(Class.forName(beastObjectName))) {
+                if (!beastObjectName.equals(name) && beastObject.getClass().isAssignableFrom(BEASTClassLoader.forName(beastObjectName))) {
                     implementations.add(beastObjectName);
                 }
             } catch (ClassNotFoundException e) {
@@ -394,7 +395,7 @@ public class DocMaker {
         buf.append("</head>\n");
         buf.append("<body>\n");
         buf.append("<h1>BEAST " + version.getVersionString() + " Documentation: " + beastObjectName + "</h1>\n");
-        BEASTObject beastObject = (BEASTObject) Class.forName(beastObjectName).newInstance();
+        BEASTObject beastObject = (BEASTObject) BEASTClassLoader.forName(beastObjectName).newInstance();
 
         // show all implementation of this plug-in
         String[] implementations = m_isa.get(beastObjectName);
@@ -422,7 +423,7 @@ public class DocMaker {
         for (Citation citation : beastObject.getCitationList()) {
             buf.append("<h2>Reference:</h2><p>" + citation.value() + "</p>\n");
             if (citation.DOI().length() > 0) {
-                buf.append("<p><a href=\"http://dx.doi.org/" + citation.DOI() + "\">doi:" + citation.DOI() + "</a></p>\n");
+                buf.append("<p><a href=\"https://doi.org/" + citation.DOI() + "\">doi:" + citation.DOI() + "</a></p>\n");
             }
         }
 
@@ -510,7 +511,7 @@ public class DocMaker {
                             Class<?> _class = (Class<?>) genericTypes2[0];
                             Object o = null;
                             try {
-                                o = Class.forName(_class.getName()).newInstance();
+                                o = BEASTClassLoader.forName(_class.getName()).newInstance();
                             } catch (Exception e) {
                             }
                             if (o != null && o instanceof BEASTObject) {
@@ -523,7 +524,7 @@ public class DocMaker {
                             Class<?> _class = genericType;
                             Object o = null;
                             try {
-                                o = Class.forName(_class.getName()).newInstance();
+                                o = BEASTClassLoader.forName(_class.getName()).newInstance();
                             } catch (Exception e) {
                             }
                             if (o != null && o instanceof BEASTObject) {
