@@ -1,40 +1,25 @@
 package beast.app.beastapp;
 
-import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.FileReader;
-import java.io.FileWriter;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.io.OutputStream;
-import java.io.UnsupportedEncodingException;
-import java.lang.reflect.InvocationTargetException;
-import java.net.URL;
-import java.net.URLDecoder;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.jar.JarEntry;
-import java.util.jar.JarInputStream;
-
-import javax.swing.JOptionPane;
-import javax.xml.parsers.DocumentBuilderFactory;
-
-import org.w3c.dom.Document;
-import org.w3c.dom.Element;
-
 import beast.app.BEASTVersion;
 import beast.app.util.Utils6;
 import beast.core.util.Log;
-import beast.util.PackageManager;
+import beast.util.BEASTClassLoader;
 import beast.util.Package;
+import beast.util.PackageManager;
 import beast.util.PackageVersion;
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+
+import javax.swing.*;
+import javax.xml.parsers.DocumentBuilderFactory;
+import java.io.*;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.net.URL;
+import java.net.URLDecoder;
+import java.util.*;
+import java.util.jar.JarEntry;
+import java.util.jar.JarInputStream;
 
 /**
  * Loads beast.jar and launches BEAST through the BEASTMain class
@@ -260,8 +245,8 @@ public class BeastLauncher {
 			if (majorVersion < 8) {
 				String JAVA_VERSION_MSG = "<html>" + app + " requires Java version at least 8<br>"
 						+ "but the current version is " + majorVersion + ".<br><br>"
-						+ "You can get Java from <a href='https://www.java.com/en/'>https://www.java.com/</a>.<br><br> "
-						+ "Continuing, but expect the unexpected.</html>";
+						+ "Quit to update Java from <br>"
+						+ "<a href='https://www.java.com/en/'>https://www.java.com/</a>.</html>";
 				if (!java.awt.GraphicsEnvironment.isHeadless()) {
 					JOptionPane.showMessageDialog(null, JAVA_VERSION_MSG);
 				} else {
@@ -269,7 +254,7 @@ public class BeastLauncher {
 					JAVA_VERSION_MSG = JAVA_VERSION_MSG.replaceAll("<[^<]*>", "");
 					System.err.println(JAVA_VERSION_MSG);
 				}
-				return true;
+				return false; // if majorVersion < 8 then fail
 			}
 		} catch (NumberFormatException e) {
 			// We only get here if the JVM does not return the expected
@@ -318,7 +303,7 @@ public class BeastLauncher {
 		return version;
 	}
 
-	protected static String getPath(boolean useStrictVersions, String beastFile) throws NoSuchMethodException, SecurityException, ClassNotFoundException, IllegalAccessException, IllegalArgumentException, InvocationTargetException, IOException {
+	public static String getPath(boolean useStrictVersions, String beastFile) throws NoSuchMethodException, SecurityException, ClassNotFoundException, IllegalAccessException, IllegalArgumentException, InvocationTargetException, IOException {
 		installBEASTPackage();
 		PackageManager.initialise();
 
@@ -557,100 +542,19 @@ public class BeastLauncher {
 
 	public static void run(String classPath, String main, String[] args) {
 		try {
-            List<String> cmd = new ArrayList<String>();
 
-            if (Utils6.isMac()) {
-            	// check whether a bundled jre is present
-            	BeastLauncher clu = new BeastLauncher();
-            	String launcherJar = clu.getClass().getProtectionDomain().getCodeSource().getLocation().getPath();            	
-            	String jreDir = URLDecoder.decode(new File(launcherJar).getParent(), "UTF-8") + "/../jre1.8.0_161/";
-            	            	
-            	if (new File(jreDir).exists()) {
-	                cmd.add(jreDir + "bin/java");
-            	}
+//            setJavaHeapAndStackSize(cmd, args);
+           
+			for (String jarFile : classPath.substring(1, classPath.length() - 1).split(":")) {
+				if (jarFile.toLowerCase().endsWith("jar")) {
+					BEASTClassLoader.classLoader.addJar(jarFile);
+				}
 			}
+		
+			Class<?> mainClass = BEASTClassLoader.forName(main);
+			Method mainMethod = mainClass.getMethod("main", String [].class);
+			mainMethod.invoke(null, (Object) args);
 
-            if (cmd.size() == 0) {
-	            if (System.getenv("JAVA_HOME") != null) {
-	                cmd.add(System.getenv("JAVA_HOME") + File.separatorChar
-	                        + "bin" + File.separatorChar + "java");
-	            } else
-	                cmd.add("java");
-            }
-
-            setJavaHeapAndStackSize(cmd, args);
-
-            if (System.getProperty("java.library.path") != null && System.getProperty("java.library.path").length() > 0) {
-            	cmd.add("-Djava.library.path=" + sanitise(System.getProperty("java.library.path")));
-            }            
-            
-            cmd.add("-cp");
-            cmd.add(classPath.substring(1, classPath.length() - 1));
-            cmd.add(main);
-
-            for (String arg : args) {
-            	if (arg != null) {
-            		cmd.add(arg);
-            	}
-            }
-
-            final ProcessBuilder pb = new ProcessBuilder(cmd);
-
-            Map<String, String> environment = pb.environment();
-            System.err.println(pb.command());
-
-            //File log = new File("log");
-            pb.redirectErrorStream(true);
-            
-            // Start the process and wait for it to finish.
-            final Process process = pb.start();
-
-            Thread inputThread = null;
-            boolean waitToExit = System.getProperty("launcher.wait.for.exit") != null; 
-            if (main.equals("beast.app.beastapp.BeastMain") && waitToExit) {
-            	inputThread = new Thread() {
-	            	public void run() {
-	                    OutputStream out = process.getOutputStream();
-	                    byte [] buf = new byte[2048];
-	            		while (true) {
-	            			try {
-			                    int ni = System.in.available();
-			                    if (ni > 0) {
-			                      int c = System.in.read(buf, 0, Math.min(ni, buf.length));
-			                      System.err.print((char) c);
-			                      out.write(buf, 0, c);
-			                      out.flush();
-			                    }	 
-	            			} catch (IOException e) {
-	            				e.printStackTrace();
-	            			}
-	                		try {
-								Thread.sleep(1000);
-							} catch (InterruptedException e) {
-								e.printStackTrace();
-							}
-	             		}
-	            	}
-	            };
-	            inputThread.start();
-            }
-
-            if (waitToExit) {
-	            int c;
-	            BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
-	            while ((c = input.read()) != -1) {
-	                System.out.print((char)c);
-	            }
-	            input.close();
-	
-	            final int exitStatus = process.waitFor();
-	            if (exitStatus != 0) {
-	            	System.err.println(process.getErrorStream());
-	            }
-	            if (main.equals("beast.app.beastapp.BeastMain")) {
-	            	inputThread.stop();
-	            }
-            }
         } catch (Exception e) {
             e.printStackTrace();
         }
@@ -672,6 +576,10 @@ public class BeastLauncher {
         		cmd.add(arg);
         		args[i] = null;
         		stackSizeSet = true;
+        	} else if (arg.startsWith("-Xss")) {
+        		cmd.add(arg);
+        		args[i] = null;
+        		stackSizeSet = true;
         	}
         }            
 
