@@ -16,6 +16,7 @@ import javax.swing.JFrame;
 import javax.swing.WindowConstants;
 import javax.swing.filechooser.FileFilter;
 
+import beagle.BeagleFactory;
 import beagle.BeagleFlag;
 import beagle.BeagleInfo;
 import beast.app.BEASTVersion2;
@@ -281,6 +282,9 @@ public class BeastMain {
                                 false, "BEAGLE: specify scaling scheme to use"),
                         new Arguments.Option("help", "Print this information and stop"),
                         new Arguments.Option("version", "Print version and stop"),
+                        new Arguments.Option("strictversions", "Use only package versions as specified in the 'required' attribute"),
+                        new Arguments.StringOption("D", "DEFINITIONS", "attribute-value pairs to be replaced in the XML, e.g., -D \"arg1=10,arg2=20\"").allowMultipleUse(),
+                        new Arguments.Option("sampleFromPrior", "samples from prior for MCMC analysis (by adding sampleFromPrior=\"true\" in the first run element)"),
                 });
 
         try {
@@ -416,6 +420,7 @@ public class BeastMain {
             }
         }
 
+        
         BeastConsoleApp consoleApp = null;
 
         final String nameString = "BEAST " + version.getVersionString();
@@ -503,6 +508,9 @@ public class BeastMain {
             	Log.err.println("No input file specified");
                 System.exit(1);
             }
+            if (dialog.useStrictVersions()) {
+            	MCMCargs.add("-strictversions");
+            }
 
         } else {
             if (arguments.hasOption("overwrite")) {
@@ -514,9 +522,32 @@ public class BeastMain {
             }
         }
 
+        if (arguments.hasOption("strictversions")) {
+        	MCMCargs.add("-strictversions");
+        }
+        
+        if (arguments.hasOption("sampleFromPrior")) {
+        	MCMCargs.add("-sampleFromPrior");
+        }
+        
+        if (arguments.hasOption("D")) {
+            MCMCargs.add("-D");
+            MCMCargs.add(arguments.getStringOption("D"));
+            for (String optionVal : arguments.getAdditionalStringOptions("D")) {
+                MCMCargs.add("-D");
+                MCMCargs.add(optionVal);
+            }
+        }
+
         if (beagleShowInfo) {
-            BeagleInfo.printResourceList();
-            System.exit(0);
+            Log.info.println("\n--- BEAGLE RESOURCES ---\n");
+            for (beagle.ResourceDetails details : BeagleFactory.getResourceDetails())
+                Log.info.println(details.toString());
+
+            if (window)
+                return;
+            else
+                System.exit(0);
         }
 
         if (inputFile == null) {
@@ -549,7 +580,7 @@ public class BeastMain {
         }
 
         if (inputFile != null && inputFile.getParent() != null && working) {
-            System.setProperty("file.name.prefix", inputFile.getParentFile().getAbsolutePath());
+            System.setProperty("file.name.prefix", inputFile.getParentFile().getAbsolutePath() + File.separator);
         }
 
         if (window) {
