@@ -1,6 +1,7 @@
 package beast.app.beauti;
 
 
+
 import beast.app.BEASTVersion2;
 import beast.app.beauti.BeautiDoc.ActionOnExit;
 import beast.app.beauti.BeautiDoc.DOC_STATUS;
@@ -15,8 +16,10 @@ import beast.core.BEASTInterface;
 import beast.core.util.Log;
 import beast.evolution.alignment.Alignment;
 import beast.math.distributions.MRCAPrior;
+import beast.util.BEASTClassLoader;
 import beast.util.PackageManager;
 import jam.framework.DocumentFrame;
+
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
@@ -37,6 +40,7 @@ import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.PrintStream;
+import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.net.URL;
@@ -55,6 +59,11 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
      * current directory for opening files *
      */
     public static String g_sDir = System.getProperty("user.dir");
+    
+    public static void setCurrentDir(String currentDir) {
+    	g_sDir = currentDir;
+    	Utils.saveBeautiProperty("currentDir", currentDir);
+    }
 
 	/**
      * File extension for Beast specifications
@@ -107,6 +116,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         this.doc = doc;
         this.doc.addBeautiDocListener(this);
         doc.setBeauti(this);
+        BEAUtiIntances++;
     }
 
     void setTitle() {
@@ -147,13 +157,14 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
     Action a_template = new ActionTemplate();
     Action a_managePackages = new ActionManagePacakges();
     Action a_clearClassPath = new ActionClearClassPath();
-    Action a_appLauncher = new ActionAppLauncher();
+    
 //    public Action a_import = new ActionImport();
     public Action a_save = new ActionSave();
     Action a_saveas = new ActionSaveAs();
     Action a_close = new ActionClose();
     Action a_quit = new ActionQuit();
     Action a_viewall = new ActionViewAllPanels();
+    Action a_appLauncher = new ActionLaunch();
 
     Action a_help = new ActionHelp();
     Action a_msgs = new ActionMsgs();
@@ -187,7 +198,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             if (docStatus != DOC_STATUS.DIRTY) {
                 if (docStatus == DOC_STATUS.NO_DOCUMENT)
                     JOptionPane.showMessageDialog(null,
-                            "There is no data to save to file");
+                            "The model is empty, there is nothing to save. Set up a model (by importing data) before saving.");
 
                 return;
             }
@@ -224,7 +235,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
     boolean saveAs() {
         if (doc.validateModel() == DOC_STATUS.NO_DOCUMENT) {
             JOptionPane.showMessageDialog(null,
-                    "There is no data to save to file");
+                    "The model is empty, there is nothing to save. Set up a model (by importing data) before saving.");
             return false;
         }
         String fileSep = System.getProperty("file.separator");
@@ -247,8 +258,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
             // f.getAbsoluteFile().toString()+"\"");
             doc.setFileName(file.getAbsolutePath());// fc.getSelectedFile().toString();
             if (doc.getFileName().lastIndexOf(fileSep) > 0) {
-                g_sDir = doc.getFileName().substring(0,
-                        doc.getFileName().lastIndexOf(fileSep));
+                setCurrentDir(doc.getFileName().substring(0,
+                        doc.getFileName().lastIndexOf(fileSep)));
             }
             if (!doc.getFileName().toLowerCase().endsWith(FILE_EXT) && !doc.getFileName().toLowerCase().endsWith(FILE_EXT2))
                 doc.setFileName(doc.getFileName().concat(FILE_EXT));
@@ -320,8 +331,8 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
                     fileSep = "\\\\";
                 }
                 if (doc.getFileName().lastIndexOf(fileSep) > 0) {
-                    g_sDir = doc.getFileName().substring(0,
-                            doc.getFileName().lastIndexOf(fileSep));
+                    setCurrentDir(doc.getFileName().substring(0,
+                            doc.getFileName().lastIndexOf(fileSep)));
                 }
                 try {
                 	// TODO: deal with json files
@@ -402,6 +413,19 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         } // actionPerformed
     }
     
+    class ActionLaunch extends MyAction {
+        private static final long serialVersionUID = 1;
+
+        public ActionLaunch() {
+            super("Launch Apps", "Launch BEAST Apps supplied by packages", "launch", -1);
+        } // c'tor
+
+        @Override
+		public void actionPerformed(ActionEvent ae) {
+        	AppLauncher.main(new String[]{});
+        } // actionPerformed
+    }
+
     class ActionClearClassPath extends MyAction {
         private static final long serialVersionUID = 1;
 
@@ -419,18 +443,6 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         } // actionPerformed
     }
     
-    class ActionAppLauncher extends MyAction {
-        private static final long serialVersionUID = 1;
-
-        public ActionAppLauncher() {
-            super("Launch Apps", "Launch BEAST Apps supplied by packages", "launch", -1);
-        } // c'tor
-
-        @Override
-		public void actionPerformed(ActionEvent ae) {
-        	AppLauncher.main(new String[]{});
-        } // actionPerformed
-    }
 
     //    class ActionImport extends MyAction {
 //        private static final long serialVersionUID = 1;
@@ -780,6 +792,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         helpMenu.add(a_msgs);
         helpMenu.add(a_citation);
         helpMenu.add(a_viewModel);
+        addCustomHelpMenus(helpMenu);
         if (!Utils.isMac() || Utils6.isMajorLower(Utils6.JAVA_1_8)) {
             helpMenu.add(a_about);
         }
@@ -789,7 +802,34 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
         return menuBar;
     } // makeMenuBar
 
-    private void createFileMenu() {
+    // Find sub-classes of BeautiHelpAction and add custom help menu items
+    // for these classes
+    private void addCustomHelpMenus(JMenu helpMenu) {
+        String[] PACKAGE_DIRS = {"beast.app",};
+        String helpClass = "beast.app.beauti.BeautiHelpAction";
+        List<String> helpActions = new ArrayList<>();
+        for (String packageName : PACKAGE_DIRS) {
+        	helpActions.addAll(PackageManager.find(helpClass, packageName));
+        }
+        if (helpActions.size() > 1) {
+            helpMenu.addSeparator();
+            for (String className : helpActions) {
+            	if (!className.equals(helpClass)) {
+		            try {
+		            	Class<?> _class = Class.forName(className);
+		                Constructor<?> con = _class.getConstructor(BeautiDoc.class);
+		                BeautiHelpAction helpAction = (BeautiHelpAction) con.newInstance(doc);
+		                helpMenu.add(helpAction);
+		            } catch (Throwable t) {
+		            	t.printStackTrace();
+		            }
+            	}
+            }
+            helpMenu.addSeparator();
+        }
+    }
+
+	private void createFileMenu() {
     	// first clear menu
    		fileMenu.removeAll();
 
@@ -1047,7 +1087,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 	
 						@Override
 						public void actionPerformed(ActionEvent e) {
-							g_sDir = dir;
+							setCurrentDir(dir);
 						}
 	
 		            };
@@ -1102,6 +1142,9 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
     } // hidePanels
 
     public void setUpPanels() throws NoSuchMethodException, SecurityException, ClassNotFoundException, InstantiationException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
+    	if (isInitialising) {
+    		//return;
+    	}
         isInitialising = true;
         // remove any existing tabs
         if (getTabCount() > 0) {
@@ -1158,6 +1201,13 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
     public static Beauti main2(String[] args) {
     	Utils6.startSplashScreen();
     	Utils6.logToSplashScreen("Initialising BEAUti");
+    	
+    	// retrieve previously stored working directory
+    	String currentDir = Utils.getBeautiProperty("currentDir");
+    	if (currentDir != null && currentDir.length() > 1) {
+    		g_sDir = currentDir;
+    	}
+    	
         try {
         	ByteArrayOutputStream baos = null;
             for (String arg : args) {
@@ -1222,9 +1272,14 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
             BeautiDoc doc = new BeautiDoc();
             BeautiDoc.baos = baos;
+
+            // make sure templates know we are in BEAUti while parsing arguments 
+            Beauti.BEAUtiIntances++;
             if (doc.parseArgs(args) == ActionOnExit.WRITE_XML) {
                 return null;
             }
+            // reset instances
+            Beauti.BEAUtiIntances--;
 
             final Beauti beauti = new Beauti(doc);
 
@@ -1232,7 +1287,7 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
                 // set up application about-menu for Mac
                 // Mac-only stuff
                 try {
-                    URL url = ClassLoader.getSystemResource(ModelBuilder.ICONPATH + "beauti.png");
+                    URL url = BEASTClassLoader.classLoader.getResource(ModelBuilder.ICONPATH + "beauti.png");
                     Icon icon = null;
                     if (url != null) {
                         icon = new ImageIcon(url);
@@ -1272,12 +1327,12 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
                         beast.app.util.Utils.macOSXRegistration(application);
                     else // <= Java 8
                         jam.mac.Utils.macOSXRegistration(application);
-                } catch (Exception e) {
+                } catch (Throwable e) {
                     // ignore
                 }
                 if (Utils6.isMajorLower(Utils6.JAVA_9)) {
                     try {
-                        Class<?> class_ = Class.forName("jam.maconly.OSXAdapter");
+                        Class<?> class_ = BEASTClassLoader.forName("jam.maconly.OSXAdapter");
                         Method method = class_.getMethod("enablePrefs", boolean.class);
                         method.invoke(null, false);
                     } catch (java.lang.NoSuchMethodException e) {
@@ -1335,7 +1390,6 @@ public class Beauti extends JTabbedPane implements BeautiDocListener {
 
             // check file needs to be save on closing main frame
             frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
-            BEAUtiIntances++;
             frame.addWindowListener(new WindowAdapter() {
                 @Override
 				public void windowClosing(WindowEvent e) {
