@@ -22,6 +22,7 @@ import beast.core.BEASTInterface;
 import beast.core.Input;
 import beast.core.Input.Validate;
 import beast.core.util.Log;
+import beast.util.BEASTClassLoader;
 import beast.util.PackageManager;
 
 
@@ -62,7 +63,7 @@ public class InputEditorFactory {
         	// ignore inner classes, which are marked with $
         	if (!inputEditor.contains("$")) {
 	            try {
-	                Class<?> _class = Class.forName(inputEditor);
+	                Class<?> _class = BEASTClassLoader.forName(inputEditor);
 	                
 	                
 	                Constructor<?> con = _class.getConstructor(BeautiDoc.class);
@@ -79,7 +80,7 @@ public class InputEditorFactory {
 	                }
 	            } catch (java.lang.InstantiationException e) {
 	                // ingore input editors that are inner classes
-	            } catch (Exception e) {
+	            } catch (Throwable e) {
 	                // print message
 	                Log.err.println(e.getClass().getName() + ": " + e.getMessage());
 	            }
@@ -181,10 +182,10 @@ public class InputEditorFactory {
             if (listInputEditorMap.containsKey(inputClass)) {
                 // use custom list input editor
                 String inputEditorName = listInputEditorMap.get(inputClass);
-                Constructor<?> con = Class.forName(inputEditorName).getConstructor(BeautiDoc.class);
+                Constructor<?> con = BEASTClassLoader.forName(inputEditorName).getConstructor(BeautiDoc.class);
                 inputEditor = (InputEditor) con.newInstance(doc);
 
-                //inputEditor = (InputEditor) Class.forName(inputEditor).newInstance();
+                //inputEditor = (InputEditor) BEASTClassLoader.forName(inputEditor).newInstance();
             } else {
                 // otherwise, use generic list editor
                 inputEditor = new ListInputEditor(doc);
@@ -204,7 +205,7 @@ public class InputEditorFactory {
 	            // handle BEASTObject-input with custom input editors
 	            String inputEditorName = inputEditorMap.get(inputClass2);
 	            
-	            Constructor<?> con = Class.forName(inputEditorName).getConstructor(BeautiDoc.class);
+	            Constructor<?> con = BEASTClassLoader.forName(inputEditorName).getConstructor(BeautiDoc.class);
 	            inputEditor = (InputEditor) con.newInstance(doc);
         	}
         }        	
@@ -212,9 +213,9 @@ public class InputEditorFactory {
 //            // handle BEASTObject-input with custom input editors
 //            String inputEditor = inputEditorMap.get(inputClass);
 //            
-//            Constructor<?> con = Class.forName(inputEditor).getConstructor(BeautiDoc.class);
+//            Constructor<?> con = BEASTClassLoader.forName(inputEditor).getConstructor(BeautiDoc.class);
 //            inputEditor = (InputEditor) con.newInstance(doc);
-//            //inputEditor = (InputEditor) Class.forName(inputEditor).newInstance(doc);
+//            //inputEditor = (InputEditor) BEASTClassLoader.forName(inputEditor).newInstance(doc);
 //            //} else if (inputClass.isEnum()) {
 //            //    inputEditor = new EnumInputEditor();
 //        } else {
@@ -316,7 +317,7 @@ public class InputEditorFactory {
             List<String> classes = PackageManager.find(input.getType(), "beast");
             for (String className : classes) {
                 try {
-                    Object o = Class.forName(className).newInstance();
+                    Object o = BEASTClassLoader.forName(className).newInstance();
                     if (input.canSetValue(o, parent)) {
                         beastObjectNames.add("new " + className);
                     }
