@@ -12,6 +12,9 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.InputStreamReader;
 import java.io.PrintStream;
+import java.lang.management.ManagementFactory;
+import java.lang.reflect.Method;
+import java.net.URLDecoder;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -39,8 +42,11 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 
+import beast.app.beastapp.BeastLauncher;
 import beast.app.util.Utils;
+import beast.app.util.Utils6;
 import beast.core.util.Log;
+import beast.util.BEASTClassLoader;
 import beast.util.PackageManager;
 
 
@@ -334,60 +340,31 @@ public class AppLauncher {
 
     public void runAppFromCMD(PackageApp packageApp, String[] additionalArgs) {
         try {
+
+//              setJavaHeapAndStackSize(cmd, args);
+
+			String classPath = BeastLauncher.getPath(false, null);
+             
             PackageManager.loadExternalJars();
+  			for (String jarFile : classPath.split(":")) {
+  				if (jarFile.toLowerCase().endsWith("jar")) {
+  					BEASTClassLoader.classLoader.addJar(jarFile);
+  				}
+  			}
+  		
+  			Class<?> mainClass = BEASTClassLoader.forName(packageApp.className);
+  			Method mainMethod = mainClass.getMethod("main", String [].class);
+System.err.println("About to invoke " + packageApp.className + " " + mainMethod);
+			if (additionalArgs == null) {
+				additionalArgs = new String[] {};
+			}
+System.err.println("Args:" + Arrays.toString(additionalArgs));
+  			mainMethod.invoke(null, (Object) additionalArgs);
+System.err.println("Done invoking " + packageApp.className);
 
-            List<String> cmd = new ArrayList<>();
-            if (System.getenv("JAVA_HOME") != null) {
-                cmd.add(System.getenv("JAVA_HOME") + File.separatorChar
-                        + "bin" + File.separatorChar + "java");
-            } else
-                cmd.add("java");
-            // TODO: deal with java directives like -Xmx -Xms here
-
-            if (System.getProperty("java.library.path") != null && System.getProperty("java.library.path").length() > 0) {
-            	cmd.add("-Djava.library.path=" + sanitise(System.getProperty("java.library.path")));
-            }
-            cmd.add("-cp");
-            final String strClassPath = sanitise(System.getProperty("java.class.path"));
-            cmd.add(strClassPath);
-            cmd.add(packageApp.className);
-
-            for (String arg : packageApp.getArgs()) {
-                cmd.add(arg);
-            }
-
-            if (additionalArgs != null) {
-                for (String arg : additionalArgs)
-                    cmd.add(arg);
-            }
-
-
-            final ProcessBuilder pb = new ProcessBuilder(cmd);
-
-            System.err.println(pb.command());
-
-            //File log = new File("log");
-            pb.redirectErrorStream(true);
-            
-            // Start the process and wait for it to finish.
-            final Process process = pb.start();
-            int c;
-            BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
-            while ((c = input.read()) != -1) {
-                Log.info.print((char)c);
-            }
-            input.close();
-            final int exitStatus = process.waitFor();
-
-            if (exitStatus != 0) {
-                Log.err.println(Utils.toString(process.getErrorStream()));
-            } else {
-//                System.out.println(Utils.toString(process.getInputStream()));
-            }
-
-        } catch (Exception e) {
-            e.printStackTrace();
-        }
+          } catch (Throwable e) {
+              e.printStackTrace();
+          }
 
     }
     
