@@ -65,13 +65,14 @@ public class TreeAnnotator {
 
     private static boolean forceIntegerToDiscrete = false;
 
+    static boolean processSA = true;
+
     private boolean SAmode = false;
 
-    
-    abstract class TreeSet {
-    	abstract boolean hasNext();
-    	abstract Tree next() throws IOException;
-    	abstract void reset() throws IOException;
+    public abstract class TreeSet {
+    	public abstract boolean hasNext();
+    	public abstract Tree next() throws IOException;
+    	public abstract void reset() throws IOException;
 
 
         String inputFileName;
@@ -116,7 +117,7 @@ public class TreeAnnotator {
 
     }    
     
-    class FastTreeSet extends TreeSet {
+    public class FastTreeSet extends TreeSet {
     	int current = 0;
     	Tree [] trees;
 
@@ -154,22 +155,22 @@ public class TreeAnnotator {
 		}
 
 		@Override
-		boolean hasNext() {
+		public boolean hasNext() {
 			return current < trees.length;
 		}
 
 		@Override
-		Tree next()  {
+		public Tree next()  {
 			return trees[current++];
 		}
 
 		@Override
-		void reset()  {
+		public void reset()  {
 			current = 0;
 		}
     }
     
-    class MemoryFriendlyTreeSet extends TreeSet {
+    public class MemoryFriendlyTreeSet extends TreeSet {
 //    	Tree [] trees;
     	int current = 0;
     	int lineNr;
@@ -181,7 +182,7 @@ public class TreeAnnotator {
 
         BufferedReader fin;
 
-        MemoryFriendlyTreeSet(String inputFileName, int burninPercentage) throws IOException  {
+        public MemoryFriendlyTreeSet(String inputFileName, int burninPercentage) throws IOException  {
     		this.inputFileName = inputFileName;
     		countTrees(burninPercentage);
 
@@ -190,7 +191,7 @@ public class TreeAnnotator {
 
 
     	@Override
-		void reset() throws FileNotFoundException  {
+    	public void reset() throws FileNotFoundException  {
     		current = 0;
             fin = new BufferedReader(new FileReader(new File(inputFileName)));
             lineNr = 0;
@@ -221,7 +222,7 @@ public class TreeAnnotator {
             if (str == null) {
                 return null;
             }
-            if (str.contains("[")) {
+            if (str.matches("^\\s*\\[.*")) {
                 final int start = str.indexOf('[');
                 int end = str.indexOf(']', start);
                 while (end < 0) {
@@ -252,9 +253,9 @@ public class TreeAnnotator {
 
         private void parseTreesBlock() throws IOException  {
             // read to first non-empty line within trees block
-            String str = fin.readLine().trim();
+            String str = readLine().trim();
             while (str.equals("")) {
-                str = fin.readLine().trim();
+                str = readLine().trim();
             }
 
             // if first non-empty line is "translate" then parse translate block
@@ -339,12 +340,12 @@ public class TreeAnnotator {
     	
     	
     	@Override
-		boolean hasNext() {
+    	public boolean hasNext() {
     		return current < totalTrees;
     	}
     	
     	@Override
-		Tree next() throws IOException {
+    	public Tree next() throws IOException {
 			String str = nextLine();
     		if (!isNexus) {
                 TreeParser treeParser;
@@ -472,10 +473,11 @@ public class TreeAnnotator {
         if (targetOption != Target.USER_TARGET_TREE) {
             try {
             	treeSet.reset();
+                cladeSystem.setProcessSA(false);
             	while (treeSet.hasNext()) {
             		Tree tree = treeSet.next();
                     tree.getLeafNodeCount();
-                    if (tree.getDirectAncestorNodeCount() > 0 && !SAmode) {
+                    if (tree.getDirectAncestorNodeCount() > 0 && !SAmode && processSA) {
                         SAmode = true;
                         Log.err.println("A tree with a sampled ancestor is found. Turning on\n the sampled ancestor " +
                                 "summary analysis.");
@@ -483,6 +485,7 @@ public class TreeAnnotator {
                             throw new RuntimeException("The common ancestor height is not \n available for trees with sampled " +
                                     "ancestors. Please choose \n another height summary option");
                         }
+                        cladeSystem.setProcessSA(true);
                     }
 	            	cladeSystem.add(tree, false);
 	                totalTreesUsed++;
@@ -508,17 +511,17 @@ public class TreeAnnotator {
             }
             cladeSystem.calculateCladeCredibilities(totalTreesUsed);
 
-            progressStream.println("Total trees have " + totalTrees + ", where " + totalTreesUsed + " are used.");
+            progressStream.println("Total number of trees " + totalTrees + ", where " + totalTreesUsed + " are used.");
 
             progressStream.println("Total unique clades: " + cladeSystem.getCladeMap().keySet().size());
             progressStream.println();
         }  else {
-            // even when a user specificed target tree is provided we still need to count the totalTreesUsed for subsequent steps.
+            // even when a user specified target tree is provided we still need to count the totalTreesUsed for subsequent steps.
             treeSet.reset();
             while (treeSet.hasNext()) {
                 Tree tree = treeSet.next();
                 tree.getLeafNodeCount();
-                if (tree.getDirectAncestorNodeCount() > 0 && !SAmode) {
+                if (tree.getDirectAncestorNodeCount() > 0 && !SAmode && processSA) {
                     SAmode = true;
                     Log.err.println("A tree with a sampled ancestor is found. Turning on\n the sampled ancestor " +
                             "summary analysis.");
@@ -581,7 +584,9 @@ public class TreeAnnotator {
 
         // this call increments the clade counts and it shouldn't
         // this is remedied with removeClades call after while loop below
-        cladeSystem = new CladeSystem(targetTree);
+        cladeSystem = new CladeSystem();
+        cladeSystem.setProcessSA(processSA);
+        cladeSystem.add(targetTree, true);
         int totalTreesUsedNew = 0;
         try {
             int counter = 0;
@@ -773,7 +778,7 @@ public class TreeAnnotator {
             for (int i=1; i<bits2.length(); i=i+2) {
                 bits2.set(i, false);
             }
-            if (node.isFake()) {
+            if (node.isFake() && processSA) {
                 int index = cladeSystem.getTaxonIndex(node.getDirectAncestorChild());
                 bits2.set(2 * index + 1);
             }
@@ -889,7 +894,7 @@ public class TreeAnnotator {
                             if (node.isDirectAncestor()) {
                                 node.getParent().setHeight(mean);
                             }
-                            if (node.isFake()) {
+                            if (node.isFake() && processSA) {
                                 node.getDirectAncestorChild().setHeight(mean);
                             }
                             node.setHeight(mean);
@@ -898,7 +903,7 @@ public class TreeAnnotator {
                             if (node.isDirectAncestor()) {
                                 node.getParent().setHeight(median);
                             }
-                            if (node.isFake()) {
+                            if (node.isFake() && processSA) {
                                 node.getDirectAncestorChild().setHeight(median);
                             }
                             node.setHeight(median);
@@ -1216,8 +1221,8 @@ public class TreeAnnotator {
     Set<String> attributeNames = new HashSet<>();
     TaxonSet taxa = null;
 
-    static boolean processBivariateAttributes = false;
-
+    static boolean processBivariateAttributes = true;
+    
 //    static {
 //        try {
 //            System.loadLibrary("jri");
@@ -1399,7 +1404,9 @@ public class TreeAnnotator {
                         new Arguments.Option("help", "option to print this message"),
                         new Arguments.Option("forceDiscrete", "forces integer traits to be treated as discrete traits."),
                         new Arguments.Option("lowMem", "use less memory, which is a bit slower."),
-                        new Arguments.RealOption("hpd2D", "the HPD interval to be used for the bivariate traits")
+                        new Arguments.RealOption("hpd2D", "the HPD interval to be used for the bivariate traits"),
+                        new Arguments.Option("nohpd2D", "suppress calculation of HPD intervals for the bivariate traits"),
+                        new Arguments.Option("noSA", "interpret the tree set as begin from a not being from a sampled ancestor analysis, even if there are zero branch lengths in the tree set")
                 });
 
         try {
@@ -1410,6 +1417,14 @@ public class TreeAnnotator {
             System.exit(1);
         }
 
+        if (arguments.hasOption("nohpd2D")) {
+        	processBivariateAttributes = false;
+        }
+        
+        if (arguments.hasOption("noSA")) {
+        	processSA = false;
+        }
+
         if (arguments.hasOption("forceDiscrete")) {
             Log.info.println("  Forcing integer traits to be treated as discrete traits.");
             forceIntegerToDiscrete = true;
@@ -1496,7 +1511,10 @@ public class TreeAnnotator {
 			e.printStackTrace();
 		}
 
-        System.exit(0);
+        if (args.length == 0) {
+        	// only need exit when in GUI mode
+        	System.exit(0);
+        }
     }
 
     /**
@@ -1539,7 +1557,7 @@ public class TreeAnnotator {
         cladeSystem.getTreeCladeCodes(targetTree, ctarget);
 
         // temp collecting heights inside loop allocated once
-        double[] hs = new double[clades];
+        double[][] hs = new double[clades][treeSet.totalTrees];
 
         // heights total sum from posterior trees
         double[] ths = new double[clades];
@@ -1556,12 +1574,12 @@ public class TreeAnnotator {
                 int j = postOrderList[k];
                 for (int i = 0; i < clades; ++i) {
                     if( CollectionUtils.isSubSet(ctarget[i], ctree[j]) ) {
-                        hs[i] = tree.getNode(j).getHeight();
+                        hs[i][counter] = tree.getNode(j).getHeight();
                     }
                 }
             }
             for (int k = 0; k < clades; ++k) {
-                ths[k] += hs[k];
+                ths[k] += hs[k][counter];
             }
             totalTreesUsed += 1;
             if (counter > 0 && counter % reportStepSize == 0 && reported < 61) {
@@ -1583,6 +1601,20 @@ public class TreeAnnotator {
             ths[k] /= totalTreesUsed;
             final Node node = targetTree.getNode(k);
             node.setHeight(ths[k]);
+            String attributeName = "CAheight";
+            double [] values = hs[k];
+            double min = values[0];
+            double max = values[0];
+            for (double d : values) {
+            	min = Math.min(d, min);
+            	max = Math.max(d, max);
+            }
+            if (Math.abs(min - max) > 1e-10) {
+	            annotateMeanAttribute(node, attributeName + "_mean", values);
+	            annotateMedianAttribute(node, attributeName + "_median", values);
+	            annotateHPDAttribute(node, attributeName + "_95%_HPD", 0.95, values);
+	            annotateRangeAttribute(node, attributeName + "_range", values);
+            }
         }
 
         assert (totalTreesUsed == this.totalTreesUsed);
