@@ -105,6 +105,14 @@ public class State extends BEASTObject {
      */
     private HashMap<BEASTInterface, List<BEASTInterface>> outputMap;
 
+    /**
+     * Returns a list of BEAST objects
+     * @return
+     */
+    public List<BEASTInterface> getOutputs(BEASTInterface beastInterface) {
+        return outputMap.get(beastInterface);
+    }
+
     /**
      * Same as m_outputMap, but only for StateNodes indexed by the StateNode number
      * We need this since the StateNode changes regularly, so unlike the output map
@@ -248,7 +256,7 @@ public class State extends BEASTObject {
      * @param sample chain state number
      * @return  true if stored  to disk
      */
-    public void store(final int sample) {
+    public void store(final long sample) {
         //Arrays.fill(changeStateNodes, -1);
         nrOfChangedStateNodes = 0;
     }
@@ -260,7 +268,7 @@ public class State extends BEASTObject {
      */
     public void restore() {
         for (int i = 0; i < nrOfChangedStateNodes; i++) {
-            stateNode[changeStateNodes[i]].restore();
+            stateNode[changeStateNodes[i]].restoreStateNode();
         }
     }
 
@@ -323,7 +331,7 @@ public class State extends BEASTObject {
      *
      * @param sample TODO
      */
-    public void storeToFile(final int sample) {
+    public void storeToFile(final long sample) {
         try {
             PrintStream out = new PrintStream(stateFileName + ".new");
             out.print(toXML(sample));
@@ -345,7 +353,7 @@ public class State extends BEASTObject {
      *
      * @param sample TODO*
      */
-    public String toXML(final int sample) {
+    public String toXML(final long sample) {
         final StringBuilder buf = new StringBuilder();
         buf.append("<itsabeastystatewerein version='2.0' sample='").append(sample).append("'>\n");
         for (final StateNode node : stateNode) {
@@ -402,24 +410,29 @@ public class State extends BEASTObject {
         for (int childIndex = 0; childIndex < children.getLength(); childIndex++) {
             final Node child = children.item(childIndex);
             if (child.getNodeType() == Node.ELEMENT_NODE) {
-                final String id = child.getAttributes().getNamedItem("id").getNodeValue();
-                int stateNodeIndex = 0;
-
-                // An init node without ID - should not bring the house down, does it?
-                // I have not checked if the state is restored correctly or not (JH)
-                while (stateNode[stateNodeIndex].getID() != null &&
-                        !stateNode[stateNodeIndex].getID().equals(id)) {
-                    stateNodeIndex++;
-                    if (stateNodeIndex >= stateNode.length) {
-                    	Log.warning.println("Cannot restore statenode id");
-                    	break;
-                    }
-                }
-                if (stateNodeIndex < stateNode.length) {
-	                final StateNode stateNode2 = stateNode[stateNodeIndex].copy();
-	                stateNode2.fromXML(child);
-	                stateNode[stateNodeIndex].assignFromFragile(stateNode2);
-                }
+            	Node idNode = child.getAttributes().getNamedItem("id");
+            	if (idNode != null) {
+	                final String id = idNode.getNodeValue();
+	                int stateNodeIndex = 0;
+	
+	                // An init node without ID - should not bring the house down, does it?
+	                // I have not checked if the state is restored correctly or not (JH)
+	                while (stateNode[stateNodeIndex].getID() != null &&
+	                        !stateNode[stateNodeIndex].getID().equals(id)) {
+	                    stateNodeIndex++;
+	                    if (stateNodeIndex >= stateNode.length) {
+	                    	Log.warning.println("Cannot restore statenode id " + id + " -- item is ignored");
+	                    	break;
+	                    }
+	                }
+	                if (stateNodeIndex < stateNode.length) {
+		                final StateNode stateNode2 = stateNode[stateNodeIndex].copy();
+		                stateNode2.fromXML(child);
+		                stateNode[stateNodeIndex].assignFromFragile(stateNode2);
+	                }
+            	} else {
+                	Log.warning.println("Cannot restore statenode without id -- item is ignored");
+            	}
             }
         }
     }
