@@ -164,7 +164,7 @@ public class CalibratedYuleModel extends SpeciesTreeDistribution {
                 final TaxonSet ti = taxaSets.get(i);
                 if (ti.containsAny(tk)) {
                     if (!(ti.containsAll(tk) || tk.containsAll(ti))) {
-                        throw new IllegalArgumentException("Overlapping taxaSets??");
+                        throw new IllegalArgumentException("Overlapping taxaSets " + tk.getID() + "  and " + ti.getID() + "??");
                     }
                 }
             }
