@@ -3,6 +3,7 @@ package beast.evolution.tree;
 import beast.core.*;
 import beast.core.util.Log;
 import beast.evolution.alignment.TaxonSet;
+import beast.util.BEASTClassLoader;
 import beast.util.TreeParser;
 
 import java.io.PrintStream;
@@ -173,7 +174,7 @@ public class Tree extends StateNode implements TreeInterface {
      */
     protected Node newNode() {
         try {
-            return (Node) Class.forName(nodeTypeInput.get()).newInstance();
+            return (Node) BEASTClassLoader.forName(nodeTypeInput.get()).newInstance();
         } catch (Exception e) {
             throw new RuntimeException("Cannot create node of type "
                     + nodeTypeInput.get() + ": " + e.getMessage());
