@@ -87,16 +87,29 @@ public class Prior extends Distribution {
         try {
             newx = dist.sample(1)[0];
 
-            if (x instanceof RealParameter) {
-                for (int i = 0; i < newx.length; i++) {
-                    ((RealParameter) x).setValue(i, newx[i]);
-                }
-            } else if (x instanceof IntegerParameter) {
-                for (int i = 0; i < newx.length; i++) {
-                    ((IntegerParameter) x).setValue(i, (int)Math.round(newx[i]));
-                }
+            if (newx.length == x.getDimension()) {            	
+	            if (x instanceof RealParameter) {
+	                for (int i = 0; i < newx.length; i++) {
+	                    ((RealParameter) x).setValue(i, newx[i]);
+	                }
+	            } else if (x instanceof IntegerParameter) {
+	                for (int i = 0; i < newx.length; i++) {
+	                    ((IntegerParameter) x).setValue(i, (int)Math.round(newx[i]));
+	                }
+	            }
+            } else if (newx.length == 1) {
+            	// assume it is a multi dimensional distribution with iid components
+            	for (int k = 0; k < x.getDimension(); k++) {
+		            if (x instanceof RealParameter) {
+		                ((RealParameter) x).setValue(k, newx[0]);
+		            } else if (x instanceof IntegerParameter) {
+		                ((IntegerParameter) x).setValue(k, (int)Math.round(newx[0]));
+		            }
+		            if (k < x.getDimension()-1) {
+		            	newx = dist.sample(1)[0];
+		            }
+            	}
             }
-
         } catch (MathException e) {
             e.printStackTrace();
             throw new RuntimeException("Failed to sample!");
