@@ -35,6 +35,8 @@ package beast.util;
 import beast.app.BEASTVersion;
 import beast.app.util.Arguments;
 import beast.app.util.Utils6;
+import beast.core.BEASTInterface;
+import beast.core.BEASTObjectStore;
 import beast.core.util.Log;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
@@ -51,6 +53,7 @@ import javax.xml.parsers.ParserConfigurationException;
 import java.io.*;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
+import java.net.HttpURLConnection;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
@@ -76,6 +79,9 @@ import java.util.zip.ZipFile;
 // TODO: on windows allow installation on drive D: and pick up add-ons in drive C:
 //@Description("Manage all BEAUti packages and list their dependencies")
 public class PackageManager {
+	
+
+	
     public static final BEASTVersion beastVersion = BEASTVersion.INSTANCE;
 
     public enum UpdateStatus {AUTO_CHECK_AND_ASK, AUTO_UPDATE, DO_NOT_CHECK};
@@ -474,8 +480,24 @@ public class PackageManager {
             Package thisPkg = entry.getKey();
             PackageVersion thisPkgVersion = entry.getValue();
 
-            // create directory
             URL templateURL = thisPkg.getVersionURL(thisPkgVersion);
+
+            // check the URL exists
+            HttpURLConnection huc = (HttpURLConnection) templateURL.openConnection();
+            huc.setRequestMethod("HEAD");
+            int responseCode = huc.getResponseCode();
+            if (responseCode == HttpURLConnection.HTTP_MOVED_PERM || responseCode == HttpURLConnection.HTTP_NOT_FOUND) { 
+            	// RRB: should be "if (responseCode != HttpURLConnection.HTTP_OK)"
+            	// but package file hosted on github (which are most of them)
+            	// produce a HttpURLConnection.HTTP_FORBIDDEN for some reason
+            	throw new IOException("Could not find pacakge at URL\n" + templateURL + "\n"
+            			+ "The server may be bussy, or network may be down.\n"
+            			+ "If you suspect there is a problem with the URL \n"
+            			+ "(the URL may have a typo, or the file was removed)\n"
+            			+ "please contact the package maintainer.\n");
+            }
+            
+            // create directory
             ReadableByteChannel rbc = Channels.newChannel(templateURL.openStream());
             String dirName = getPackageDir(thisPkg, thisPkgVersion, useAppDir, customDir);
             File dir = new File(dirName);
@@ -614,15 +636,18 @@ public class PackageManager {
      * 
      */
     private static void closeClassLoader() {
-    	//try {
-    		if (Utils6.isWindows()) {
+    	try {
+    		if (Utils6.isWindows() && Utils6.getMajorJavaVersion() == 8) {
+    			// this class cast exception works on java 8, but not java 9 or above
     			URLClassLoader sysLoader = (URLClassLoader) PackageManager.class.getClassLoader();
-    			// sysLoader.close(); <= only since Java 1.7
-    		}
-		//} catch (IOException e) {
-		//	Log.warning.println("Could not close ClassLoader: " + e.getMessage());
-		//}
-		
+    			// sysLoader.close(); // <= only since Java 1.7, so should be commented out for
+    			// build of launcher.jar with java 6 compatibility 
+    		}		
+    	//} catch (IOException e) {
+        //    Log.warning.println("Could not close ClassLoader: " + e.getMessage());
+        } catch (ClassCastException e) {
+            Log.warning.println("Could not close ClassLoader: " + e.getMessage());
+		}
 	}
 
 	private static void deleteRecursively(File file, List<File> deleteFailed) {
@@ -745,7 +770,7 @@ public class PackageManager {
         
         URL u;
 		try {
-			u = Class.forName("beast.app.beastapp.BeastMain").getProtectionDomain().getCodeSource().getLocation();
+			u = BEASTClassLoader.forName("beast.app.beastapp.BeastMain").getProtectionDomain().getCodeSource().getLocation();
 		} catch (ClassNotFoundException e) {
 			// e.printStackTrace();
 			return null;
@@ -1034,13 +1059,13 @@ public class PackageManager {
         // way with java 8 when the -Dbeast.load.jars=true
         // directive is given. This can be useful for developers
         // but generally slows down application starting.
-        if (Boolean.getBoolean("beast.load.jars") == false || Utils6.getMajorJavaVersion() != 8) {
-            externalJarsLoaded = true;
-        	Utils6.logToSplashScreen("PackageManager::findDataTypes");
-            findDataTypes();
-        	Utils6.logToSplashScreen("PackageManager::Done");
-    		return;
-    	}
+//        if (Boolean.getBoolean("beast.load.jars") == false || Utils6.getMajorJavaVersion() != 8) {
+//            externalJarsLoaded = true;
+//        	Utils6.logToSplashScreen("PackageManager::findDataTypes");
+//            findDataTypes();
+//        	Utils6.logToSplashScreen("PackageManager::Done");
+//    		return;
+//    	}
 
         for (String jarDirName : getBeastDirectories()) {
         	loadPackage(jarDirName);
@@ -1053,7 +1078,7 @@ public class PackageManager {
     
 	private static void findDataTypes() {
 		try {
-			Method findDataTypes = Class.forName("beast.evolution.alignment.Alignment").getMethod("findDataTypes");
+			Method findDataTypes = BEASTClassLoader.forName("beast.evolution.alignment.Alignment").getMethod("findDataTypes");
 			findDataTypes.invoke(null);
 		} catch (Exception e) {
 			// too bad, cannot load data types
@@ -1141,47 +1166,48 @@ public class PackageManager {
             if (jarDir.exists() && jarDir.isDirectory()) {
                 for (String fileName : jarDir.list()) {
                     if (fileName.endsWith(".jar")) {
-                        Log.debug.print("Probing: " + fileName + " ");
-                        // check that we are not reload existing classes
-                        String loadedClass = null;
-                        try {
-                            JarInputStream jarFile = new JarInputStream
-                                    (new FileInputStream(jarDir.getAbsolutePath() + "/" + fileName));
-                            JarEntry jarEntry;
-
-                            while (loadedClass == null) {
-                                jarEntry = jarFile.getNextJarEntry();
-                                if (jarEntry == null) {
-                                    break;
-                                }
-                                if ((jarEntry.getName().endsWith(".class"))) {
-                                    String className = jarEntry.getName().replaceAll("/", "\\.");
-                                    className = className.substring(0, className.lastIndexOf('.'));
-                                    try {
-                                        /*Object o =*/
-                                        Class.forName(className);
-                                        loadedClass = className;
-                                    } catch (Exception e) {
-                                        // TODO: handle exception
-                                    }
-                                    if (loadedClass == null && packageNameAndVersion != null) {
-                                        classToPackageMap.put(className, packageNameAndVersion);
-                                    }
-                                }
-                            }
-                            jarFile.close();
-                        } catch (Exception e) {
-                            e.printStackTrace();
-                        }
-
-
-                        @SuppressWarnings("deprecation")
-                        URL url = new File(jarDir.getAbsolutePath() + "/" + fileName).toURL();
-                        if (loadedClass == null) {
+//                        Log.debug.print("Probing: " + fileName + " ");
+//                        // check that we are not reload existing classes
+//                        String loadedClass = null;
+//                        try {
+//                            JarInputStream jarFile = new JarInputStream
+//                                    (new FileInputStream(jarDir.getAbsolutePath() + "/" + fileName));
+//                            JarEntry jarEntry;
+//
+//                            while (loadedClass == null) {
+//                                jarEntry = jarFile.getNextJarEntry();
+//                                if (jarEntry == null) {
+//                                    break;
+//                                }
+//                                if ((jarEntry.getName().endsWith(".class"))) {
+//                                    String className = jarEntry.getName().replaceAll("/", "\\.");
+//                                    className = className.substring(0, className.lastIndexOf('.'));
+//                                    try {
+//                                        /*Object o =*/
+//                                    	BEASTClassLoader.forName(className);
+//                                        loadedClass = className;
+//                                    } catch (Throwable e) {
+//                                        // TODO: handle exception
+//                                    	e.printStackTrace();
+//                                    }
+//                                    if (loadedClass == null && packageNameAndVersion != null) {
+//                                        classToPackageMap.put(className, packageNameAndVersion);
+//                                    }
+//                                }
+//                            }
+//                            jarFile.close();
+//                        } catch (Exception e) {
+//                            e.printStackTrace();
+//                        }
+//
+//
+//                        @SuppressWarnings("deprecation")
+                        URL url = new File(jarDir.getAbsolutePath() + "/" + fileName).toURI().toURL();
+//                        if (loadedClass == null) {
                             addURL(url);
-                        } else {
-                            Log.debug.println("Skip loading " + url + ": contains class " + loadedClass + " that is already loaded");
-                        }
+//                        } else {
+//                            Log.debug.println("Skip loading " + url + ": contains class " + loadedClass + " that is already loaded");
+//                        }
                     }
                 }
             }
@@ -1337,30 +1363,31 @@ public class PackageManager {
      * @throws IOException if something goes wrong when adding a url
      */
     public static void addURL(URL u) throws IOException {
-        // ClassloaderUtil clu = new ClassloaderUtil();
-        PackageManager clu = new PackageManager();
-        // URLClassLoader sysLoader = (URLClassLoader)
-        // ClassLoader.getSystemClassLoader();
-        URLClassLoader sysLoader = (URLClassLoader) clu.getClass().getClassLoader();
-        URL urls[] = sysLoader.getURLs();
-        for (URL url : urls) {
-            if (url.toString().toLowerCase().equals(u.toString().toLowerCase())) {
-                Log.debug.println("URL " + u + " is already in the CLASSPATH");
-                return;
-            }
-        }
-        Class<?> sysclass = URLClassLoader.class;
-        try {
-            // Parameters
-            Class<?>[] parameters = new Class[]{URL.class};
-            Method method = sysclass.getDeclaredMethod("addURL", parameters);
-            method.setAccessible(true);
-            method.invoke(sysLoader, u);
-            Log.debug.println("Loaded URL " + u);
-        } catch (Throwable t) {
-            t.printStackTrace();
-            throw new IOException("Error, could not add URL to system classloader");
-        }
+    	BEASTClassLoader.classLoader.addURL(u);
+//        // ClassloaderUtil clu = new ClassloaderUtil();
+//        PackageManager clu = new PackageManager();
+//        // URLClassLoader sysLoader = (URLClassLoader)
+//        // ClassLoader.getSystemClassLoader();
+//        URLClassLoader sysLoader = (URLClassLoader) clu.getClass().getClassLoader();
+//        URL urls[] = sysLoader.getURLs();
+//        for (URL url : urls) {
+//            if (url.toString().toLowerCase().equals(u.toString().toLowerCase())) {
+//                Log.debug.println("URL " + u + " is already in the CLASSPATH");
+//                return;
+//            }
+//        }
+//        Class<?> sysclass = URLClassLoader.class;
+//        try {
+//            // Parameters
+//            Class<?>[] parameters = new Class[]{URL.class};
+//            Method method = sysclass.getDeclaredMethod("addURL", parameters);
+//            method.setAccessible(true);
+//            method.invoke(sysLoader, u);
+//            Log.debug.println("Loaded URL " + u);
+//        } catch (Throwable t) {
+//            t.printStackTrace();
+//            throw new IOException("Error, could not add URL to system classloader");
+//        }
         String classpath = System.getProperty("java.class.path");
         String jar = u + "";
         classpath += System.getProperty("path.separator") + jar.substring(5);
@@ -1539,7 +1566,7 @@ public class PackageManager {
         result = new ArrayList<String>();
 
         try {
-            cls = Class.forName(classname);
+            cls = BEASTClassLoader.forName(classname);
             result = find(cls, pkgnames);
         } catch (Exception e) {
             e.printStackTrace();
@@ -1563,7 +1590,7 @@ public class PackageManager {
         result = new ArrayList<String>();
 
         try {
-            cls = Class.forName(classname);
+            cls = BEASTClassLoader.forName(classname);
             result = find(cls, pkgname);
         } catch (Exception e) {
             e.printStackTrace();
@@ -1636,7 +1663,7 @@ public class PackageManager {
             if (className.startsWith(pkgname)) {
                 //Log.debug.println(className);
                 try {
-                    Class<?> clsNew = Class.forName(className);
+                    Class<?> clsNew = BEASTClassLoader.forName(className);
 
                     // no abstract classes
                     if (!Modifier.isAbstract(clsNew.getModifiers()) &&
@@ -1960,6 +1987,28 @@ public class PackageManager {
     	return classToPackageMap;
     }
 
+    /** return set of Strings in the format of classToPackageMap (like "bModelTest v0.3.2")
+     * for all packages used by o and its predecessors in the model graph.
+     */
+    public static Set<String> getPackagesAndVersions(Object o) {
+    	Set<String> packagesAndVersions = new LinkedHashSet<String>();
+    	getPackagesAndVersions(o, packagesAndVersions);
+    	return packagesAndVersions;
+    }
+    
+    /** traverse model graph starting at o, and collect packageAndVersion strings
+     * along the way.
+     */
+    private static void getPackagesAndVersions(Object o, Set<String> packagesAndVersions) {
+    	String packageAndVersion = classToPackageMap.get(o.getClass().getName());
+    	if (packageAndVersion != null) {
+    		packagesAndVersions.add(packageAndVersion);
+    	}
+    	for (BEASTInterface o2 : BEASTObjectStore.listActiveBEASTObjects(o)) {
+    		getPackagesAndVersions(o2, packagesAndVersions);
+    	}
+	}
+    
     private static void initPackageMap(String jarDirName) {
         try {
             File versionFile = new File(jarDirName + "/version.xml");
