@@ -156,6 +156,7 @@ public class TreeParser extends Tree implements StateNodeInitialiser {
             setRoot(parseNewick(newickInput.get()));
         }
 
+        
         super.initAndValidate();
         m_sTaxaNames = null;
 
@@ -225,9 +226,33 @@ public class TreeParser extends Tree implements StateNodeInitialiser {
         }
 
         initStateNodes();
+
+        // check if all labels are in the Newick tree
+        if (labels != null) {
+        	boolean [] done = new boolean[labels.size()];
+        	checkAllLabelsAreCovered(getRoot(), done);
+            for (int i = 0; i < done.length; i++) {
+            	if (!done[i]) {
+            		Log.warning("WARNING: Taxon " + labels.get(i) + " was not found in the Newick tree");
+            	}
+            }        	
+        }
     } // init
 
-    public TreeParser() {
+    private void checkAllLabelsAreCovered(Node node, boolean[] done) {
+    	if (node.isLeaf()) {
+			int i = labels.indexOf(node.getID());
+			if (i >=0 && i < done.length) {
+				done[i] = true;
+			}
+    	} else {
+    		for (Node child : node.getChildren()) {
+    			checkAllLabelsAreCovered(child, done);
+    		}
+    	}
+	}
+
+	public TreeParser() {
     }
 
     public TreeParser(final Alignment alignment, final String newick) {
@@ -486,8 +511,17 @@ public class TreeParser extends Tree implements StateNodeInitialiser {
 
                             value = arrayValues;
                         } catch (NumberFormatException ex) {
-                            throw new TreeParsingException("Encountered vector-valued metadata entry with " +
-                                    "one or more non-numeric elements.");
+                        	//throw new TreeParsingException("Encountered vector-valued metadata entry with " +
+                        	//              "one or more non-numeric elements.");
+                        	
+                        	// it is a non-numerical vector -- store as String
+                            List<NewickParser.AttribValueContext> elementContexts = attribctx.attribValue().vector().attribValue();
+
+                            String[] arrayValues = new String[elementContexts.size()];
+                            for (int i = 0; i < elementContexts.size(); i++)
+                                arrayValues[i] = elementContexts.get(i).getText();
+
+                            value = arrayValues;
                         }
 
                     } else
