@@ -3,34 +3,22 @@ package test.beast.app.beauti;
 
 
 
-import static org.fest.swing.edt.GuiActionRunner.execute;
-import static org.fest.swing.finder.JFileChooserFinder.findFileChooser;
-
-import java.awt.Component;
-import java.io.File;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
-
-import javax.swing.JComboBox;
-
+import beast.app.util.Utils;
 import org.fest.swing.data.Index;
 import org.fest.swing.data.TableCell;
 import org.fest.swing.edt.GuiTask;
-import org.fest.swing.fixture.JButtonFixture;
-import org.fest.swing.fixture.JCheckBoxFixture;
-import org.fest.swing.fixture.JComboBoxFixture;
-import org.fest.swing.fixture.JFileChooserFixture;
-import org.fest.swing.fixture.JOptionPaneFixture;
-import org.fest.swing.fixture.JTabbedPaneFixture;
-import org.fest.swing.fixture.JTableCellFixture;
-import org.fest.swing.fixture.JTableFixture;
+import org.fest.swing.fixture.*;
 import org.fest.swing.image.ScreenshotTaker;
 import org.junit.Test;
 
-import beast.app.util.Utils;
-import test.beast.beast2vs1.TestFramework;
-import test.beast.beast2vs1.trace.Expectation;
+import javax.swing.*;
+import java.awt.*;
+import java.awt.event.KeyEvent;
+import java.io.File;
+import java.util.Arrays;
+
+import static org.fest.swing.edt.GuiActionRunner.execute;
+import static org.fest.swing.finder.JFileChooserFinder.findFileChooser;
 
 
 
@@ -83,14 +71,19 @@ public class BeautiRateTutorialTest extends BeautiBase {
 		printTableContents(t);
 		checkTableContents(t, 
 				"[RSV2_1, RSV2, 129, 209, nucleotide, RSV2_1, RSV2_1, RSV2_1, false]*" +
-				"[RSV2_2, RSV2, 129, 210, nucleotide, RSV2_2, RSV2_2, RSV2_2, false]*" +
-				"[RSV2_3, RSV2, 129, 210, nucleotide, RSV2_3, RSV2_3, RSV2_3, false]"
+				"[RSV2_2, RSV2, 129, 210, nucleotide, RSV2_2, RSV2_2, RSV2_1, false]*" +
+				"[RSV2_3, RSV2, 129, 210, nucleotide, RSV2_3, RSV2_3, RSV2_1, false]"
 			);
 		printBeautiState(f);
-		assertStateEquals("Tree.t:RSV2_2", "clockRate.c:RSV2_2", "birthRate.t:RSV2_2", "Tree.t:RSV2_3", "clockRate.c:RSV2_3", "birthRate.t:RSV2_3", "Tree.t:RSV2_1", "birthRate.t:RSV2_1");
-		assertOperatorsEqual("StrictClockRateScaler.c:RSV2_2", "YuleBirthRateScaler.t:RSV2_2", "YuleModelTreeScaler.t:RSV2_2", "YuleModelTreeRootScaler.t:RSV2_2", "YuleModelUniformOperator.t:RSV2_2", "YuleModelSubtreeSlide.t:RSV2_2", "YuleModelNarrow.t:RSV2_2", "YuleModelWide.t:RSV2_2", "YuleModelWilsonBalding.t:RSV2_2", "strictClockUpDownOperator.c:RSV2_2", "StrictClockRateScaler.c:RSV2_3", "YuleBirthRateScaler.t:RSV2_3", "YuleModelTreeScaler.t:RSV2_3", "YuleModelTreeRootScaler.t:RSV2_3", "YuleModelUniformOperator.t:RSV2_3", "YuleModelSubtreeSlide.t:RSV2_3", "YuleModelNarrow.t:RSV2_3", "YuleModelWide.t:RSV2_3", "YuleModelWilsonBalding.t:RSV2_3", "strictClockUpDownOperator.c:RSV2_3", "YuleBirthRateScaler.t:RSV2_1", "YuleModelTreeScaler.t:RSV2_1", "YuleModelTreeRootScaler.t:RSV2_1", "YuleModelUniformOperator.t:RSV2_1", "YuleModelSubtreeSlide.t:RSV2_1", "YuleModelNarrow.t:RSV2_1", "YuleModelWide.t:RSV2_1", "YuleModelWilsonBalding.t:RSV2_1");
-		assertPriorsEqual("YuleModel.t:RSV2_1", "YuleModel.t:RSV2_2", "YuleModel.t:RSV2_3", "ClockPrior.c:RSV2_2", "YuleBirthRatePrior.t:RSV2_2", "ClockPrior.c:RSV2_3", "YuleBirthRatePrior.t:RSV2_3", "YuleBirthRatePrior.t:RSV2_1");
-		assertTraceLogEqual("posterior", "likelihood", "prior", "treeLikelihood.RSV2_2", "TreeHeight.t:RSV2_2", "clockRate.c:RSV2_2", "YuleModel.t:RSV2_2", "birthRate.t:RSV2_2", "treeLikelihood.RSV2_3", "TreeHeight.t:RSV2_3", "clockRate.c:RSV2_3", "YuleModel.t:RSV2_3", "birthRate.t:RSV2_3", "treeLikelihood.RSV2_1", "TreeHeight.t:RSV2_1", "YuleModel.t:RSV2_1", "birthRate.t:RSV2_1");
+//		assertStateEquals("Tree.t:RSV2_2", "clockRate.c:RSV2_2", "birthRate.t:RSV2_2", "Tree.t:RSV2_3", "clockRate.c:RSV2_3", "birthRate.t:RSV2_3", "Tree.t:RSV2_1", "birthRate.t:RSV2_1");
+//		assertOperatorsEqual("StrictClockRateScaler.c:RSV2_2", "YuleBirthRateScaler.t:RSV2_2", "YuleModelTreeScaler.t:RSV2_2", "YuleModelTreeRootScaler.t:RSV2_2", "YuleModelUniformOperator.t:RSV2_2", "YuleModelSubtreeSlide.t:RSV2_2", "YuleModelNarrow.t:RSV2_2", "YuleModelWide.t:RSV2_2", "YuleModelWilsonBalding.t:RSV2_2", "strictClockUpDownOperator.c:RSV2_2", "StrictClockRateScaler.c:RSV2_3", "YuleBirthRateScaler.t:RSV2_3", "YuleModelTreeScaler.t:RSV2_3", "YuleModelTreeRootScaler.t:RSV2_3", "YuleModelUniformOperator.t:RSV2_3", "YuleModelSubtreeSlide.t:RSV2_3", "YuleModelNarrow.t:RSV2_3", "YuleModelWide.t:RSV2_3", "YuleModelWilsonBalding.t:RSV2_3", "strictClockUpDownOperator.c:RSV2_3", "YuleBirthRateScaler.t:RSV2_1", "YuleModelTreeScaler.t:RSV2_1", "YuleModelTreeRootScaler.t:RSV2_1", "YuleModelUniformOperator.t:RSV2_1", "YuleModelSubtreeSlide.t:RSV2_1", "YuleModelNarrow.t:RSV2_1", "YuleModelWide.t:RSV2_1", "YuleModelWilsonBalding.t:RSV2_1");
+//		assertPriorsEqual("YuleModel.t:RSV2_1", "YuleModel.t:RSV2_2", "YuleModel.t:RSV2_3", "ClockPrior.c:RSV2_2", "YuleBirthRatePrior.t:RSV2_2", "ClockPrior.c:RSV2_3", "YuleBirthRatePrior.t:RSV2_3", "YuleBirthRatePrior.t:RSV2_1");
+//		assertTraceLogEqual("posterior", "likelihood", "prior", "treeLikelihood.RSV2_2", "TreeHeight.t:RSV2_2", "clockRate.c:RSV2_2", "YuleModel.t:RSV2_2", "birthRate.t:RSV2_2", "treeLikelihood.RSV2_3", "TreeHeight.t:RSV2_3", "clockRate.c:RSV2_3", "YuleModel.t:RSV2_3", "birthRate.t:RSV2_3", "treeLikelihood.RSV2_1", "TreeHeight.t:RSV2_1", "YuleModel.t:RSV2_1", "birthRate.t:RSV2_1");
+
+		assertStateEquals("Tree.t:RSV2_1", "clockRate.c:RSV2_2", "birthRate.t:RSV2_1", "clockRate.c:RSV2_3");
+		assertOperatorsEqual("StrictClockRateScaler.c:RSV2_2", "YuleBirthRateScaler.t:RSV2_1", "strictClockUpDownOperator.c:RSV2_2", "YuleModelTreeScaler.t:RSV2_1", "YuleModelTreeRootScaler.t:RSV2_1", "YuleModelUniformOperator.t:RSV2_1", "YuleModelSubtreeSlide.t:RSV2_1", "YuleModelNarrow.t:RSV2_1", "YuleModelWide.t:RSV2_1", "YuleModelWilsonBalding.t:RSV2_1", "StrictClockRateScaler.c:RSV2_3", "strictClockUpDownOperator.c:RSV2_3");
+		assertPriorsEqual("YuleModel.t:RSV2_1", "ClockPrior.c:RSV2_2", "YuleBirthRatePrior.t:RSV2_1", "ClockPrior.c:RSV2_3");
+		assertTraceLogEqual("posterior", "likelihood", "prior", "treeLikelihood.RSV2_2", "treeLikelihood.RSV2_3", "TreeHeight.t:RSV2_1", "clockRate.c:RSV2_2", "YuleModel.t:RSV2_1", "birthRate.t:RSV2_1", "treeLikelihood.RSV2_1", "clockRate.c:RSV2_3");
 	
 		//1a. Link trees... 
 		warning("1a. Link trees...");
@@ -110,6 +103,7 @@ public class BeautiRateTutorialTest extends BeautiBase {
 		cell.startEditing();
 		comboBox.selectAllText();
 		comboBox.enterText("tree");
+		comboBox.pressAndReleaseKeys(KeyEvent.VK_ENTER);
 		cell.stopEditing();
 		printBeautiState(f);
 		assertStateEquals("clockRate.c:RSV2_2", "clockRate.c:RSV2_3", "Tree.t:tree", "birthRate.t:tree");
@@ -134,6 +128,7 @@ public class BeautiRateTutorialTest extends BeautiBase {
 		cell.startEditing();
 		comboBox.selectAllText();
 		comboBox.enterText("clock");
+		comboBox.pressAndReleaseKeys(KeyEvent.VK_ENTER);
 		cell.stopEditing();
 		printBeautiState(f);
 		assertStateEquals("Tree.t:tree", "birthRate.t:tree");
@@ -235,68 +230,12 @@ public class BeautiRateTutorialTest extends BeautiBase {
 		//7. Run MCMC and look at results in Tracer, TreeAnnotator->FigTree
 		warning("7. Run MCMC and look at results in Tracer, TreeAnnotator->FigTree");
 		makeSureXMLParses();
-		
-		// TODO: this should run as a separate process since the BEAUti run can interfere with the BEAST run on Hudson
- 		//MEPRunner runner = new MEPRunner(org.fest.util.Files.temporaryFolder());
- 		//runner.analyse(0);
-		
+
 		long t1 = System.currentTimeMillis();
 		System.err.println("total time: " + (t1 - t0)/1000 + " seconds");
 		
 	}
-	
-	
-	 // This is for debugging the test only
-// 	MEPRunner should be run from MEPTutorial()
-// 	@Test
-// 	public void runXML() throws Exception {
-// 		//System.setProperty("file.name.prefix", org.fest.util.Files.temporaryFolder().getAbsolutePath());
-// 		MEPRunner runner = new MEPRunner(org.fest.util.Files.temporaryFolder());
-// 		runner.analyse(0);
-// 		
-// 	}
-    
-	class MEPRunner extends TestFramework {
-		
-		MEPRunner(File file) {
-			super();
-			setUp(new String[]{"/x.xml"});
-			dirName = file.getPath();
-			logDir = "";
-			useSeed = false;
-	        checkESS = false;
-			testFile = "RSV2";
-		}
 
-		@Override
-		protected List<Expectation> giveExpectations(int index_XML) throws Exception {
-	        List<Expectation> expList = new ArrayList<Expectation>();
-	        addExpIntoList(expList,"posterior", -6131.89, 0.922052);
-	        addExpIntoList(expList,"likelihood", -5496.28, 0.401133);
-	        // low ESS for seed=128
-	        addExpIntoList(expList,"prior", -635.603, 1.215535);
-	        addExpIntoList(expList,"treeLikelihood.1", -1440.16, 0.197223);
-	        addExpIntoList(expList,"treeLikelihood.3", -2271.52, 0.300608);
-	        addExpIntoList(expList,"treeLikelihood.2", -1784.59, 0.29738);
-	        addExpIntoList(expList,"TreeHeight", 56.06136, 0.125308);
-	        addExpIntoList(expList,"kappa.1", 7.727761, 0.069897);
-	        addExpIntoList(expList,"kappa.2", 10.41839, 0.093578);
-	        addExpIntoList(expList,"kappa.3", 11.97769, 0.090429);
-	        addExpIntoList(expList,"mutationRate.1", 0.698603, 0.000868);
-	        addExpIntoList(expList,"mutationRate.2", 0.960092, 0.00099);
-	        addExpIntoList(expList,"mutationRate.3", 1.33987, 0.001132);
-	        addExpIntoList(expList,"clockRate", 0.002179, 1.16E-5);
-	        addExpIntoList(expList,"popSize", 37.44745, 0.368656);
-	        // low ESS for seed=128
-	        addExpIntoList(expList,"CoalescentConstant", -590.862, 1.164024);
-			return expList;
-		}
-		
-	}
-    
-
-
-	
 	@Test
 	public void MEPBSPTutorial() throws InterruptedException {
 		if (true) {return;}
@@ -355,7 +294,7 @@ public class BeautiRateTutorialTest extends BeautiBase {
 		printBeautiState(f);
 		
 		// 4. set chain-length to 10M, log every 10K
-		warning("4. set chaing-length to 10M, log every 10K");
+		warning("4. set chain-length to 10M, log every 10K");
 		f = f.selectTab("MCMC");
 		beautiFrame.textBox("chainLength").selectAll().setText("10000000");
 		beautiFrame.button("tracelog.editButton").click();
